import "core-js/modules/es.array.join";
import _mapInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/map";
import _forEachInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/for-each";
import _everyInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/every";
import _Array$isArray from "@babel/runtime-corejs3/core-js-stable/array/is-array";
import _assertThisInitialized from "@babel/runtime-corejs3/helpers/esm/assertThisInitialized";
import _inheritsLoose from "@babel/runtime-corejs3/helpers/esm/inheritsLoose";
import _defineProperty from "@babel/runtime-corejs3/helpers/esm/defineProperty";
import Builder from './Builder';
import typeOf from './typeOf';

var UnionBuilder = function (_Builder) {
  _inheritsLoose(UnionBuilder, _Builder);

  function UnionBuilder(builders, defaultValue) {
    var _this;

    _this = _Builder.call(this, 'union', defaultValue) || this;

    _defineProperty(_assertThisInitialized(_this), "builders", []);

    if ("production" !== process.env.NODE_ENV) {
      _this.invariant(_Array$isArray(builders) && builders.length > 0 && _everyInstanceProperty(builders).call(builders, function (builder) {
        return builder instanceof Builder;
      }), 'A non-empty array of blueprints are required for a union.');

      _this.builders = builders;

      _this.addCheck(_this.checkUnions, builders);
    }

    return _this;
  }

  var _proto = UnionBuilder.prototype;

  _proto.checkUnions = function checkUnions(path, value, builders) {
    var _this2 = this;

    if ("production" !== process.env.NODE_ENV) {
      var usage = {};
      var keys = [];
      var type = typeOf(value);

      _forEachInstanceProperty(builders).call(builders, function (builder) {
        if (usage[builder.type]) {
          _this2.invariant(false, "Multiple instances of \"" + builder.type + "\" are not supported.", path);
        } else if (builder.type === 'union') {
          _this2.invariant(false, 'Nested unions are not supported.', path);
        } else {
          usage[builder.type] = true;
          keys.push(builder.typeAlias());
        }
      });

      if (usage.shape && usage.object) {
        this.invariant(false, 'Objects and shapes within the same union are not supported.', path);
      }

      var checked = false;

      _forEachInstanceProperty(builders).call(builders, function (builder) {
        if (type === builder.type || type === 'object' && builder.type === 'shape' || builder.type === 'custom') {
          checked = true;
          builder.runChecks(path, value, _this2.currentStruct, _this2.options);
        }
      });

      this.invariant(checked, "Type must be one of: " + keys.join(', '), path);
    }
  };

  _proto.typeAlias = function typeAlias() {
    var _context;

    return _mapInstanceProperty(_context = this.builders).call(_context, function (builder) {
      return builder.typeAlias();
    }).join(' | ');
  };

  return UnionBuilder;
}(Builder);

export { UnionBuilder as default };
export function union(builders, defaultValue) {
  return new UnionBuilder(builders, defaultValue);
}